#!/usr/bin/env python
# coding: utf-8

# Copyright (c) [2024] [Zekeriya Ender Eger, Pinar Acar]

# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:

# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.

# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.



# IMPORT LIBRARIES NUMPY FOR NUMERICAL OPERATIONS AND MATPLOTLIB FOR VISULAIZATION

# In[1]:


import numpy as np
import matplotlib.pyplot as plt


# LOAD ANY DATA WITH np.load, MAGNETIZATION IS USED TO PLOT M VS FE PLOTS, UNCERTAINTY DATA DO NOT REQUIRE EXTRA INFORMATION

# In[14]:


ENGNTN = np.load('Deterministic/KBT28H05.npy') # Deterministic Values
Magnetization=np.load('Magnetization.npy') # Magnetization for Deterministic Values
ENGNTNkbt=np.load('T_Uncertainty/KBT28H00_T_UNCcomb643.npy') # Uncertainty Values


# CALCULATE THE MEAN FIELD THEORY RESULTS

# In[18]:


k_bT=2.8 # Temperature
h=0.5 # External Magnetic Field
N=4 # Lattice Size
M=np.arange(-1,1.01,0.01) # Magnetization for MFT
Num=N*N
J=1 # Interaction Strength
q=4 # Coordination Number
F=((N-1)/N)*Num*k_bT*(-J*q/(2*k_bT))*M*M-Num*h*M+k_bT*Num*((0.5*(1+M))*np.log((0.5*(1.0001+M)))+(0.5*(1-M))*np.log((0.5*(1.0001-M))))


# FIND THE AVERAGE VALUES FOR NUMERICAL CALCULATIONS

# In[19]:


avgcombinsT=np.zeros(len(np.unique(Magnetization)))
uniqs=np.unique(Magnetization)
for i in range(len(uniqs)):
    avgcombinsT[i]=ENGNTN[Magnetization==uniqs[i]].sum()/len(ENGNTN[Magnetization==uniqs[i]])


# PLOT M VS FE CURVE

# In[24]:


plt.figure(figsize=(8,6))
M1=np.unique(Magnetization)
plt.plot(M,F/np.abs(F[0]),color='red',label='Mean Field Energy')
plt.scatter(Magnetization,ENGNTN/np.abs(ENGNTN[-1]), marker="v")
plt.plot(M1,avgcombinsT/np.abs(avgcombinsT[0]),color='yellow',label='Average Energy')
plt.legend(fontsize=13)
plt.title('Magnetization vs Normalized Free Energy')
plt.xlabel('Magnetization',fontsize=15)
plt.ylabel('Normalized Free Energy',fontsize=15)
plt.xticks(fontsize=13)
plt.yticks(fontsize=14)
fig=plt.gcf()
plt.show()
filename = 'figures/' + 'Magnetization_vs_NFE' +'.png'
plt.savefig(filename)


# PLOT UNCERTAINTY

# In[23]:


plt.figure(figsize=(8,6))
plt.hist(ENGNTNkbt/np.abs(ENGNTNkbt[-1]),bins=500,color='green',alpha=0.7)
plt.title('Uncertainty of $k_BT$')
plt.xlabel('Normalized Free Energy',fontsize=15)
plt.ylabel('Frequency',fontsize=15)
plt.xticks(fontsize=13)
plt.yticks(fontsize=14)
fig=plt.gcf()
plt.show()
filename = 'figures/' + 'Uncertainty_of_k_BT' +'.png'
plt.savefig(filename)


# In[ ]:




